-- Here are the configurable settings.
creative_mode_config = {}

-- Miscellaneous settings
creative_mode_config.misc =
{	
	-- Whether the following settings should be preserved for the same game once they are loaded.
	-- Preserved settings will be applied during the same game and not affected by future update of this mod, unless otherwise specified.
	-- If this is set to false, every time a save is loaded, the game reads this file to get the values.
	-- Please note that not all settings can be preserved due to Factorio's mechanism. The behaviour for each setting is written inside [] in its description.
	-- Even if you set this value to true, you can still type /c remote.call("creative-mode", "update_config") to update the settings.
	-- [Preserved]
	preserve_settings = true,
	
	-- Whether Creative Mode will be automatically enabled with all cheats for every new game. This doesn't affect existing game that did not have this mod before.
	-- [Not preserved]
	auto_enable = false,
	
	-- Whether a popup will be shown to the first player in every new game or any existing game that did not have this mod before asking for enabling Creative Mode.
	-- [Not preserved]
	show_popup_in_new_game = true,
	-- Whether a hint about the lua command will be shown after the option for enabling Creative Mode is denied.
	-- [Not preserved]
	show_hint_after_creative_mode_denied = false
}

-- Settings about the item source.
creative_mode_config.item_source =
{
	-- Whether the item source will insert the items into vehicle's inventory if the vehicle is in front of it and there is no transport belt.
	-- [Preserved]
	can_insert_to_vehicle = true,

	-- Whether the item source will insert the items into player's inventory if the player is in front of it and there is no transport belt or vehicle.
	-- [Preserved]
	can_insert_to_player = true,
	
	-- Whether the item source will insert only the items only once into player's inventory if the player is standing in front of it. To insert more items, player has to move away and go back or change the filter.
	-- If it is set to false, the item source will insert items into player's inventory in every tick.
	-- Only works when can_insert_to_player is true.
	-- [Preserved]
	insert_only_once_to_player = true,
	
	-- The amount of item to be inserted into player's inventory in each time.
	-- Only works when insert_only_once_to_player is true.
	-- [Preserved]
	insert_to_player_amount = 1,
	
	-- Whether the value of insert_to_player_amount means the number of stacks.
	-- If it is set to false, it means the number of items instead.
	-- [Preserved]
	insert_to_player_by_stack = true,
	
	-- Whether the item source will drop the items on ground if there is no entity in front of it.
	-- [Preserved]
	can_drop_on_ground = false,
}

-- Settings about the item void.
creative_mode_config.item_void =
{
	-- Whether the item void will remove the items from vehicle's inventory if the vehicle is in front of it and there is no transport belt.
	-- [Preserved]
	can_remove_from_vehicle = false,

	-- Whether the item void will remove the items from player's inventory if the player is in front of it and there is no transport belt or vehicle.
	-- [Preserved]
	can_remove_from_player = false,
	
	-- Whether the item void will remove the items on ground in front of it if there is no other entity in front of it.
	-- [Preserved]
	can_remove_from_ground = true,
}

-- Settings about the duplicating chest and duplicating provider chest.
creative_mode_config.duplicating_provider_chest =
{
	-- Inventory size of duplicating chest and duplicating provider chest.
	-- Possible value: 1 - 255
	-- [Not preserved]
	-- [Need restart Factorio]
	inventory_size = 48,
}

-- Settings about the void requester chest.
creative_mode_config.void_requester_chest =
{
	-- Inventory size of void requester chest.
	-- Possible value: 1 - 255
	-- [Not preserved]
	-- [Need restart Factorio]
	inventory_size = 48,
}

-- Settings about the creative chest and creative provider chest.
creative_mode_config.creative_provider_chest =
{
	-- Inventory size of creative chest and creative provider chest.
	-- Possible value: 1 - 254
	-- (Factorio only supports maximum inventory size of 255, and we use 1 extra slot as the trash slot. So the maximum value becomes 254.)
	-- Its value affects how many chests you need to place for one item cycle. Larger inventory size means fewer chests are needed.
	-- However, it is also more likely to cause performance issue when hovering the chests.
	-- Changing this value for an existing game will cause item regrouping.
	-- [Not preserved]
	-- [Need restart Factorio]
	inventory_size = 150,
	
	-- Whether it should contain the hidden items. Some items are hidden for good reasons, like the rocket-part item which is not reasonable if you get it.
	-- Changing this value for an existing game will cause item regrouping.
	-- [Preserved]
	contain_hidden_items = false,
}

-- Settings about the duplicator.
creative_mode_config.duplicator =
{
	-- Whether the duplicator can duplicate items in vehicle's inventory if the vehicle is in front of it and there is no transport belt.
	-- [Preserved]
	can_duplicate_in_vehicle = true,
	
	-- Whether the duplicator can duplicate items in player's inventory if the player is in front of it and there is no transport belt or vehicle.
	-- [Preserved]
	can_duplicate_in_player = false,
}