-- This file contains variables and functions related to GUI.
if not gui then gui = {} end

-- Possible main popup content types. Used in show_main_popup function.
gui.main_popup_content_type =
{
	enable_creative_mode = 1,
	permanent_disable_creative_mode = -1,
	
	enable_more_cheats = 2,
	disable_more_cheats = -2
}

-- Shows a generic yes/no popup with elements of given names for the given player.
-- @param	warning_label_name		Optional. Name of the warning label. No warning label will be shown if it is not provided.
-- @param	warning_label_caption	Optional. Caption for the warning label. No warning label will be shown if it is not provided.
local function show_yes_no_popup(player, frame_name, frame_caption, btn_flow_name, yes_btn_name, no_btn_name, warning_label_name, warning_label_caption)
	local center = player.gui.center
	center.add{type = "frame", name = frame_name, direction = "vertical", caption = {frame_caption}}
	local popup = center[frame_name]
	
	if warning_label_caption then
		popup.add{type = "label", name = warning_label_name, caption = {warning_label_caption}}
		popup[warning_label_name].style.font_color = {r = 0.98, g = 0.66, b = 0.22}
	end
	
	popup.add{type = "flow", name = btn_flow_name, direction = "horizontal"}
	local buttons_flow = popup[btn_flow_name]
	
	if yes_btn_name then
		buttons_flow.add{type = "button", name = yes_btn_name, caption = creative_mode_defines.names.gui_captions.yes}
	end
	if no_btn_name then
		buttons_flow.add{type = "button", name = no_btn_name, caption = creative_mode_defines.names.gui_captions.no}
	end
end

-- Shows the more cheats popup for enabling or disabling the cheats.
local function show_more_cheats_popup(player, enable)
	-- Prepare the captions.
	local popup_caption
	local label_caption
	local cheat_mode_caption
	local loaders_caption
	local tech_caption
	local always_day_caption
	local reach_distance_caption
	local mining_speed_caption
	local running_speed_caption
	local instant_blueprint_caption
	local instant_blueprint_tooltip
	local instant_deconstruction_caption
	local instant_deconstruction_tooltip
	if enable then
		popup_caption = {"gui.creative-mode_cheats-to-enable"}
		label_caption = {"gui.creative-mode_enable-more-cheats"}
		cheat_mode_caption = {"gui.creative-mode_enable-cheat-mode"}
		loaders_caption = {"gui.creative-mode_enable-loaders"}
		tech_caption = {"gui.creative-mode_research-all-technologies"}
		always_day_caption = {"gui.creative-mode_make-all-surfaces-always-day"}
		reach_distance_caption = {"gui.creative-mode_increase-character-reach"}
		mining_speed_caption = {"gui.creative-mode_increase-character-mining-speed"}
		running_speed_caption = {"gui.creative-mode_increase-character-running-speed"}
		instant_blueprint_caption = {"gui.creative-mode_enable-instant-blueprint"}
		instant_blueprint_tooltip = {"gui.creative-mode_enable-instant-blueprint-tooltip"}
		instant_deconstruction_caption = {"gui.creative-mode_enable-instant-deconstruction"}
		instant_deconstruction_tooltip = {"gui.creative-mode_enable-instant-deconstruction-tooltip"}
	else
		popup_caption = {"gui.creative-mode_cheats-to-disable"}
		label_caption = {"gui.creative-mode_disable-more-cheats"}
		cheat_mode_caption = {"gui.creative-mode_disable-cheat-mode"}
		loaders_caption = {"gui.creative-mode_disable-loaders"}
		tech_caption = {"gui.creative-mode_reset-all-technologies"}
		always_day_caption = {"gui.creative-mode_bring-back-night-to-surfaces"}
		reach_distance_caption = {"gui.creative-mode_reset-character-reach"}
		mining_speed_caption = {"gui.creative-mode_reset-character-mining-speed"}
		running_speed_caption = {"gui.creative-mode_reset-character-running-speed"}
		instant_blueprint_caption = {"gui.creative-mode_disable-instant-blueprint"}
		instant_blueprint_tooltip = {"gui.creative-mode_disable-instant-blueprint-tooltip"}
		instant_deconstruction_caption = {"gui.creative-mode_disable-instant-deconstruction"}
		instant_deconstruction_tooltip = {"gui.creative-mode_disable-instant-deconstruction-tooltip"}
	end

	local center = player.gui.center
	center.add{type = "frame", name = creative_mode_defines.names.gui.popup, direction = "vertical", caption = popup_caption}
	local popup = center[creative_mode_defines.names.gui.popup]
	
	popup.add{type = "table", name = creative_mode_defines.names.gui.cheats_table, colspan  = 2}
	local cheats_table = popup[creative_mode_defines.names.gui.cheats_table]
		
	-- Label, and the select all button.
	cheats_table.add{type = "label", name = creative_mode_defines.names.gui.cheats_label, caption = label_caption}
	cheats_table.add{type = "button", name = creative_mode_defines.names.gui.cheats_select_all_btn, caption = {"gui.creative-mode_all"}}
	-- Cheat mode.
	cheats_table.add{type = "label", name = creative_mode_defines.names.gui.cheats_cheat_mode_label, caption = cheat_mode_caption, tooltip = {"gui.creative-mode_cheat-mode-tooltip"}}
	cheats_table.add{type = "checkbox", name = creative_mode_defines.names.gui.cheats_cheat_mode_checkbox, state = true}
	-- Loaders.
	cheats_table.add{type = "label", name = creative_mode_defines.names.gui.cheats_loaders_label, caption = loaders_caption, tooltip = {"gui.creative-mode_loaders-tooltip"}}
	cheats_table.add{type = "checkbox", name = creative_mode_defines.names.gui.cheats_loaders_checkbox, state = true}
	-- Technologies.
	cheats_table.add{type = "label", name = creative_mode_defines.names.gui.cheats_technologies_label, caption = tech_caption}
	cheats_table.add{type = "checkbox", name = creative_mode_defines.names.gui.cheats_technologies_checkbox, state = true}
	-- Always day.
	cheats_table.add{type = "label", name = creative_mode_defines.names.gui.cheats_always_day_label, caption = always_day_caption}
	cheats_table.add{type = "checkbox", name = creative_mode_defines.names.gui.cheats_always_day_checkbox, state = true}
	-- Character reach.
	cheats_table.add{type = "label", name = creative_mode_defines.names.gui.cheats_character_reach_label, caption = reach_distance_caption, tooltip = {"gui.creative-mode_no-effect-in-god-mode-tooltip"}}
	cheats_table.add{type = "checkbox", name = creative_mode_defines.names.gui.cheats_character_reach_checkbox, state = true}
	-- Character mining speed.
	cheats_table.add{type = "label", name = creative_mode_defines.names.gui.cheats_character_mining_speed_label, caption = mining_speed_caption, tooltip = {"gui.creative-mode_no-effect-in-god-mode-tooltip"}}
	cheats_table.add{type = "checkbox", name = creative_mode_defines.names.gui.cheats_character_mining_speed_checkbox, state = true}
	-- Character running speed.
	cheats_table.add{type = "label", name = creative_mode_defines.names.gui.cheats_character_running_speed_label, caption = running_speed_caption, tooltip = {"gui.creative-mode_no-effect-in-god-mode-tooltip"}}
	cheats_table.add{type = "checkbox", name = creative_mode_defines.names.gui.cheats_character_running_speed_checkbox, state = true}
	-- Instant blueprint.
	cheats_table.add{type = "label", name = creative_mode_defines.names.gui.cheats_instant_blueprint_label, caption = instant_blueprint_caption, tooltip = instant_blueprint_tooltip}
	cheats_table.add{type = "checkbox", name = creative_mode_defines.names.gui.cheats_instant_blueprint_checkbox, state = true}
	-- Instant deconstruction.
	cheats_table.add{type = "label", name = creative_mode_defines.names.gui.cheats_instant_deconstruction_label, caption = instant_deconstruction_caption, tooltip = instant_deconstruction_tooltip}
	cheats_table.add{type = "checkbox", name = creative_mode_defines.names.gui.cheats_instant_deconstruction_checkbox, state = true}
	
	
	-- Styles.
	cheats_table[creative_mode_defines.names.gui.cheats_label].style.bottom_padding = 20
	cheats_table[creative_mode_defines.names.gui.cheats_select_all_btn].style.font = "default-small-bold"
	cheats_table[creative_mode_defines.names.gui.cheats_instant_deconstruction_label].style.bottom_padding = 20
	cheats_table[creative_mode_defines.names.gui.cheats_instant_deconstruction_checkbox].style.bottom_padding = 20
	
	-- Done button.
	if enable then
		popup.add{type = "button", name = creative_mode_defines.names.gui.cheats_enable_done, caption = {"gui.creative-mode_done"}}
	else
		popup.add{type = "button", name = creative_mode_defines.names.gui.cheats_disable_done, caption = {"gui.creative-mode_done"}}
	end
end

-- Shows the main popup for the given player.
function gui.show_main_popup(player, main_popup_content_type)
	if main_popup_content_type == gui.main_popup_content_type.enable_creative_mode then
		-- Enable Creative Mode?
		global.creative_mode.has_asked_for_enable = true
		show_yes_no_popup(player,
			creative_mode_defines.names.gui.popup,
			"gui.creative-mode_ask-enable-creative-mode",
			creative_mode_defines.names.gui.popup_buttons_flow,
			creative_mode_defines.names.gui.enable_creative_mode_yes,
			creative_mode_defines.names.gui.enable_creative_mode_no,
			nil)
	elseif main_popup_content_type == gui.main_popup_content_type.permanent_disable_creative_mode then
		-- Permanently disable Creative Mode?
		show_yes_no_popup(player,
			creative_mode_defines.names.gui.popup,
			"gui.creative-mode_ask-permanently-disable-creative-mode",
			creative_mode_defines.names.gui.popup_buttons_flow,
			creative_mode_defines.names.gui.permanent_disable_creative_mode_yes,
			creative_mode_defines.names.gui.permanent_disable_creative_mode_no,
			creative_mode_defines.names.gui.popup_label,
			"gui.creative-mode_permanently-disable-creative-mode-info")
	
	elseif main_popup_content_type == gui.main_popup_content_type.enable_more_cheats then
		-- Enable more cheats?
		show_more_cheats_popup(player, true)
	elseif main_popup_content_type == gui.main_popup_content_type.disable_more_cheats then
		-- Disable more cheats?
		show_more_cheats_popup(player, false)
	end
end

-- Destroys the main popup for the given player.
local function destroy_main_popup(player)
	player.gui.center[creative_mode_defines.names.gui.popup].destroy()
end

-- Selects or desleects the checkboxes of cheats according to their states on the cheats popup.
local function select_all_cheats(popup)
	local checkbox_container = popup[creative_mode_defines.names.gui.cheats_table]
	local checkboxes = {}
	table.insert(checkboxes, checkbox_container[creative_mode_defines.names.gui.cheats_cheat_mode_checkbox])
	table.insert(checkboxes, checkbox_container[creative_mode_defines.names.gui.cheats_loaders_checkbox])
	table.insert(checkboxes, checkbox_container[creative_mode_defines.names.gui.cheats_technologies_checkbox])
	table.insert(checkboxes, checkbox_container[creative_mode_defines.names.gui.cheats_always_day_checkbox])
	table.insert(checkboxes, checkbox_container[creative_mode_defines.names.gui.cheats_character_reach_checkbox])
	table.insert(checkboxes, checkbox_container[creative_mode_defines.names.gui.cheats_character_mining_speed_checkbox])
	table.insert(checkboxes, checkbox_container[creative_mode_defines.names.gui.cheats_character_running_speed_checkbox])
	table.insert(checkboxes, checkbox_container[creative_mode_defines.names.gui.cheats_instant_blueprint_checkbox])
	table.insert(checkboxes, checkbox_container[creative_mode_defines.names.gui.cheats_instant_deconstruction_checkbox])
	
	-- Find whether we should select or deselect them.
	local should_select = false
	for _, cb in ipairs(checkboxes) do
		if not cb.state then
			should_select = true
			break
		end
	end
	
	-- Action.
	for _, cb in ipairs(checkboxes) do
		cb.state = should_select
	end
end

-- Creates or destroys the GUI of given entity on the given player.
-- Nothing will be created if there is no GUI for the entity.
function gui.create_or_destroy_gui_of_entity(player, entity, is_create)
	local left = player.gui.left
	-- Register or deregister the entity.
	if is_create then
		global.creative_mode.player_opened_entities[player.index] = entity
	else
		global.creative_mode.player_opened_entities[player.index] = nil
	end
	
	if is_create then
		-- Create GUI.
		local entity_name = entity.name
		-- Get the data about the button that we will created.
		local button_name
		if entity_name == creative_mode_defines.names.entities.creative_chest or entity_name == creative_mode_defines.names.entities.creative_provider_chest then
			-- Button for the creative chest family.
			button_name = creative_mode_defines.names.gui.creative_chest_open_button
		else
			-- No GUI for the entity.
			return
		end
		
		-- Container.
		left.add{type = "table", name = creative_mode_defines.names.gui.entity_gui_container, colspan = 2}
		local container = left[creative_mode_defines.names.gui.entity_gui_container]
		container.style.top_padding = 8
		container.style.horizontal_spacing = 0
		
		-- Button container.
		container.add{type = "flow", name = creative_mode_defines.names.gui.entity_gui_button_container}
		local button_container = container[creative_mode_defines.names.gui.entity_gui_button_container]
		
		-- Button.
		button_container.add{type = "sprite-button", name = button_name, sprite = "item/" .. entity_name, style = "button_style"}
		local button = button_container[button_name]
		button.style.minimal_width = 36
		button.style.maximal_width = 36
		button.style.minimal_height = 36
		button.style.maximal_height = 36
	else
		-- Destroy GUI.
		global.creative_mode.player_opened_entities[player.index] = nil
		local container = left[creative_mode_defines.names.gui.entity_gui_container]
		if container then
			container.destroy()
		end
	end
end

-- Creates the GUI frame for the given entity if the frame has not been created. Otherwise, destroys the frame.
-- Returns the frame if it is created.
local function create_or_destroy_entity_gui_frame(entity_gui_container, entity)
	if entity_gui_container[creative_mode_defines.names.gui.entity_gui_frame] then
		entity_gui_container[creative_mode_defines.names.gui.entity_gui_frame].destroy()
		return nil
	end
	
	entity_gui_container.add{type = "frame", name = creative_mode_defines.names.gui.entity_gui_frame, direction = "vertical", caption = entity.localised_name}
	return entity_gui_container[creative_mode_defines.names.gui.entity_gui_frame]
end

-- Returns the caption for the creative chest group number label according to the given group number.
local function get_caption_for_creative_chest_group_number_label(group_number)
	return group_number .. "/" .. global.creative_mode.creative_provider_chest_num_in_cycle
end

-- Creates the filter table for the given creative chest GUI frame according to the given chest data and group number.
local function create_filter_table_for_creative_chest_gui_frame(chest_data, group_number, frame)
	-- Slot table wrapped inside a scroll-pane.
	frame.add{type = "scroll-pane", name = creative_mode_defines.names.gui.creative_chest_filter_scroll_pane}
	local scroll_pane = frame[creative_mode_defines.names.gui.creative_chest_filter_scroll_pane]
	scroll_pane.style.maximal_height = 465
	scroll_pane.add{type = "table", name = creative_mode_defines.names.gui.creative_chest_filter_table, colspan = 10, style = "slot_table_style"}
	local filter_table = scroll_pane[creative_mode_defines.names.gui.creative_chest_filter_table]
	-- The slots.
	local start_item_index, end_item_index = creative_chest_util.get_start_end_item_index_for_group(group_number)
	local inventory = chest_data.entity.get_output_inventory()
	local slot = 1
	local inventory_size = #inventory
	local filtered_slots = chest_data.filtered_slots
	local filtered_slots_count = #filtered_slots
	local next_filtered_slot_to_check = 1
	if filtered_slots_count <= 0 then
		next_filtered_slot_to_check = 0
	end
	for i = start_item_index, end_item_index, 1 do
		local item = global.item_list[i]
		local style
		-- Check whether the slot has been filtered out.
		if next_filtered_slot_to_check <= 0 or filtered_slots[next_filtered_slot_to_check] ~= slot then
			style = "slot_button_style"
		else
			style = "red_slot_button_style"
			-- Check for the next index of filtered slot if possible.
			if filtered_slots_count >= next_filtered_slot_to_check + 1 then
				next_filtered_slot_to_check = next_filtered_slot_to_check + 1
			else
				next_filtered_slot_to_check = 0
			end
		end
		
		filter_table.add{type = "sprite-button", name = creative_mode_defines.names.gui.creative_chest_filter_slot_prefix .. slot, sprite = "item/" .. item.name, style = style, tooltip = item.localised_name}
		
		-- Next slot.
		slot = slot + 1
		-- Though it is unlikely to happen... (caused by mod version change?) Break the loop if slot exceeds.
		if slot > inventory_size then
			break
		end
	end
end

-- Adds the group number of the given creative chest and updates the GUI about its contents.
local function add_creative_chest_group_number_and_update_gui(button, creative_chest_entity, add_amount)
	local chest_groups = creative_chest_util.get_creative_chest_data_groups(creative_chest_entity)
	local chest_data, group_number, index_in_group = creative_chest_util.get_creative_chest_data_group_number(creative_chest_entity, chest_groups)
	if chest_data and group_number > 0 then
		-- Change group number.
		local new_group_number = group_number + add_amount
		new_group_number = util.repeat_index(new_group_number, 1, global.creative_mode.creative_provider_chest_num_in_cycle)
		if creative_chest_util.change_creative_chest_group_number(chest_data, chest_groups, group_number, index_in_group, new_group_number) then
			-- Update GUI.
			local item_group_container = button.parent
			-- Item group number label.
			item_group_container[creative_mode_defines.names.gui.creative_chest_item_group_number_label].caption = get_caption_for_creative_chest_group_number_label(new_group_number)
			
			-- Item group container -> frame
			local frame = item_group_container.parent
			local scroll_pane = frame[creative_mode_defines.names.gui.creative_chest_filter_scroll_pane]
			-- Destroy the original scroll pane and re-create it and its contents.
			scroll_pane.destroy()
			create_filter_table_for_creative_chest_gui_frame(chest_data, new_group_number, frame)
		end
	end
end

-- Callback of the on_gui_click event, which is invoked when a GUI element is clicked by any player.
function gui.on_gui_click(event)
	local element = event.element
	local element_name = element.name
	local player = game.players[element.player_index]
	local popup = player.gui.center[creative_mode_defines.names.gui.popup]
	
	local entity = global.creative_mode.player_opened_entities[element.player_index]
	local entity_gui_container = player.gui.left[creative_mode_defines.names.gui.entity_gui_container]
	
	if element_name == creative_mode_defines.names.gui.enable_creative_mode_yes then
		-- Enable Creative Mode? Yes
		destroy_main_popup(player)
		cheats.enable_or_disable_creative_mode(true, false, false, player)
	elseif element_name == creative_mode_defines.names.gui.enable_creative_mode_no then
		-- Enable Creative Mode? No
		destroy_main_popup(player)
		if global.creative_mode.config.misc.show_hint_after_creative_mode_denied then
			player.print({"message.creative-mode_creative-mode_denied", command_enable})
		end
		
	elseif element_name == creative_mode_defines.names.gui.permanent_disable_creative_mode_yes then
		-- Permanently disable Creative Mode? Yes
		destroy_main_popup(player)
		cheats.enable_or_disable_creative_mode(false, true, false, player)
	elseif element_name == creative_mode_defines.names.gui.permanent_disable_creative_mode_no then
		-- Permanently disable Creative Mode? No
		destroy_main_popup(player)
		
	---------------------------------------------------------------------------------------------------
		
	elseif element_name == creative_mode_defines.names.gui.cheats_select_all_btn then
		-- Select all cheats.
		select_all_cheats(popup)
		
	elseif element_name == creative_mode_defines.names.gui.cheats_enable_done then
		-- Enable cheats done.
		local checkbox_container = popup[creative_mode_defines.names.gui.cheats_table]
		if checkbox_container[creative_mode_defines.names.gui.cheats_cheat_mode_checkbox].state then
			cheats.enable_or_disable_cheat_mode(true)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_loaders_checkbox].state then
			cheats.enable_or_disable_loaders(true)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_technologies_checkbox].state then
			cheats.research_or_reset_all_technologies(true)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_always_day_checkbox].state then
			cheats.enable_or_disable_always_day(true)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_character_reach_checkbox].state then
			cheats.increase_or_reset_character_reach(true)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_character_mining_speed_checkbox].state then
			cheats.increase_or_reset_character_mining_speed(true)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_character_running_speed_checkbox].state then
			cheats.increase_or_reset_character_running_speed(true)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_character_running_speed_checkbox].state then
			cheats.increase_or_reset_character_running_speed(true)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_instant_blueprint_checkbox].state then
			cheats.enable_or_disable_instant_blueprint(true)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_instant_deconstruction_checkbox].state then
			cheats.enable_or_disable_instant_deconstruction(true)
		end
		destroy_main_popup(player)
		
	elseif element_name == creative_mode_defines.names.gui.cheats_disable_done then
		-- Disable cheats done.
		local checkbox_container = popup[creative_mode_defines.names.gui.cheats_table]
		if checkbox_container[creative_mode_defines.names.gui.cheats_cheat_mode_checkbox].state then
			cheats.enable_or_disable_cheat_mode(false)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_loaders_checkbox].state then
			cheats.enable_or_disable_loaders(false)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_technologies_checkbox].state then
			cheats.research_or_reset_all_technologies(false)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_always_day_checkbox].state then
			cheats.enable_or_disable_always_day(false)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_character_reach_checkbox].state then
			cheats.increase_or_reset_character_reach(false)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_character_mining_speed_checkbox].state then
			cheats.increase_or_reset_character_mining_speed(false)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_character_running_speed_checkbox].state then
			cheats.increase_or_reset_character_running_speed(false)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_instant_blueprint_checkbox].state then
			cheats.enable_or_disable_instant_blueprint(false)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_instant_deconstruction_checkbox].state then
			cheats.enable_or_disable_instant_deconstruction(false)
		end
		destroy_main_popup(player)
		
	---------------------------------------------------------------------------------------------------
		
	elseif element_name == creative_mode_defines.names.gui.creative_chest_open_button then
		-- Creative chest open button.
		local chest_data, group_number = creative_chest_util.get_creative_chest_data_group_number(entity, creative_chest_util.get_creative_chest_data_groups(entity))
		if chest_data and group_number > 0 then
			-- Frame.
			local frame = create_or_destroy_entity_gui_frame(entity_gui_container, entity)
			
			if frame then
				-- Item group label.
				frame.add{type = "table", name = creative_mode_defines.names.gui.creative_chest_item_group_container, colspan = 4}
				local item_group_container = frame[creative_mode_defines.names.gui.creative_chest_item_group_container]
				item_group_container.add{type = "label", name = creative_mode_defines.names.gui.creative_chest_item_group_label, style = creative_mode_defines.names.gui_styles.creative_chest_item_group_label, caption = {"gui.creative-mode_item-group"}, tooltip = {"gui.creative-mode_item-group-tooltip"}}
				-- Item group left button.
				item_group_container.add{type = "button", name = creative_mode_defines.names.gui.creative_chest_item_group_left_button, style = creative_mode_defines.names.gui_styles.creative_chest_item_group_left_right_button, caption = "<"}
				-- Item group number label.
				item_group_container.add{type = "label", name = creative_mode_defines.names.gui.creative_chest_item_group_number_label, style = creative_mode_defines.names.gui_styles.creative_chest_item_group_number_label, caption = get_caption_for_creative_chest_group_number_label(group_number)}
				-- Item group right button.
				item_group_container.add{type = "button", name = creative_mode_defines.names.gui.creative_chest_item_group_right_button, style = creative_mode_defines.names.gui_styles.creative_chest_item_group_left_right_button, caption = ">"}
				
				-- Filter items label.
				frame.add{type = "table", name = creative_mode_defines.names.gui.entity_gui_label_container, colspan = 3}
				local label_container = frame[creative_mode_defines.names.gui.entity_gui_label_container]
				label_container.add{type = "label", name = creative_mode_defines.names.gui.entity_gui_label, style = creative_mode_defines.names.gui_styles.creative_chest_select_slot_label, caption = {"gui.creative-mode_select-slots-to-filter"}}
				-- Inventory display mode button.
				local display_mode_style
				local display_mode_tooltip
				if chest_data.inventory_display_mode == creative_chest_util.inventory_display_modes.original_mode then
					display_mode_style = creative_mode_defines.names.gui_styles.creative_chest_display_mode_button_original
					display_mode_tooltip = creative_mode_defines.names.gui_captions.creative_chest_display_mode_original
				else
					display_mode_style = creative_mode_defines.names.gui_styles.creative_chest_display_mode_button_compact
					display_mode_tooltip = creative_mode_defines.names.gui_captions.creative_chest_display_mode_compact
				end
				label_container.add{type = "button", name = creative_mode_defines.names.gui.creative_chest_display_mode_button, style = display_mode_style, tooltip = display_mode_tooltip}
				-- Toggle all button.
				label_container.add{type = "button", name = creative_mode_defines.names.gui.creative_chest_toggle_all_button, style = creative_mode_defines.names.gui_styles.inventory_toggle_all_button, tooltip = {"gui.creative-mode_toggle-all-filtered-slots-tooltip"}}
				
				-- Filter table.
				create_filter_table_for_creative_chest_gui_frame(chest_data, group_number, frame)
			end
		end
	elseif element_name == creative_mode_defines.names.gui.creative_chest_item_group_left_button then
		-- Reduce group number by 1 for creative chest.
		add_creative_chest_group_number_and_update_gui(element, entity, -1)
	elseif element_name == creative_mode_defines.names.gui.creative_chest_item_group_right_button then
		-- Increase group number by 1 for creative chest.
		add_creative_chest_group_number_and_update_gui(element, entity, 1)
	elseif element_name == creative_mode_defines.names.gui.creative_chest_display_mode_button then
		-- Toggle inventory display mode for creative chest.
		local chest_data, group_number = creative_chest_util.get_creative_chest_data_group_number(entity, creative_chest_util.get_creative_chest_data_groups(entity))
		if chest_data and group_number > 0 then
			local display_mode_style
			local display_mode_tooltip
			if chest_data.inventory_display_mode == creative_chest_util.inventory_display_modes.original_mode then
				chest_data.inventory_display_mode = creative_chest_util.inventory_display_modes.compact_mode
				display_mode_style = creative_mode_defines.names.gui_styles.creative_chest_display_mode_button_compact
				display_mode_tooltip = creative_mode_defines.names.gui_captions.creative_chest_display_mode_compact
			else
				chest_data.inventory_display_mode = creative_chest_util.inventory_display_modes.original_mode
				display_mode_style = creative_mode_defines.names.gui_styles.creative_chest_display_mode_button_original
				display_mode_tooltip = creative_mode_defines.names.gui_captions.creative_chest_display_mode_original
			end
			element.style = display_mode_style
			element.tooltip = display_mode_tooltip
		end	
	elseif element_name == creative_mode_defines.names.gui.creative_chest_toggle_all_button then
		-- Toggle all button for creative chest.
		local chest_data, group_number = creative_chest_util.get_creative_chest_data_group_number(entity, creative_chest_util.get_creative_chest_data_groups(entity))
		if chest_data and group_number > 0 then
			-- Toggle all button -> label container -> frame
			local filter_table = element.parent.parent[creative_mode_defines.names.gui.creative_chest_filter_scroll_pane][creative_mode_defines.names.gui.creative_chest_filter_table]
			local start_item_index, end_item_index = creative_chest_util.get_start_end_item_index_for_group(group_number)
			local item_count = end_item_index - start_item_index + 1
			local all_on = #chest_data.filtered_slots >= item_count
			local style
			-- The array of filtered slots is renewed not matter the slots are all filtered or restored.
			chest_data.filtered_slots = {}
			if all_on then
				-- Restore all slots.
				style = "slot_button_style"
			else
				-- Filter out all slots.
				style = "red_slot_button_style"
				local inventory = chest_data.entity.get_output_inventory()
				for i = 1, item_count, 1 do
					table.insert(chest_data.filtered_slots, i)
					inventory[i].clear()
				end
			end
			for i = 1, item_count, 1 do
				filter_table[creative_mode_defines.names.gui.creative_chest_filter_slot_prefix .. i].style = style
			end
		end
	
	---------------------------------------------------------------------------------------------------
	
	else
		local slot = string.match(element.name, creative_mode_defines.match_patterns.gui.creative_chest_filter_slot)
		if slot ~= nil then
			-- Creative chest filter slot.
			slot = tonumber(slot)
			local chest_data, group_number = creative_chest_util.get_creative_chest_data_group_number(entity, creative_chest_util.get_creative_chest_data_groups(entity))
			if chest_data and group_number > 0 then
				local style
				
				for index, filtered_slot in ipairs(chest_data.filtered_slots) do
					if slot == filtered_slot then
						-- The slot was filtered. Now turn it back.
						table.remove(chest_data.filtered_slots, index)
						element.style = "slot_button_style"
						return
					end
				end
				
				-- The slot was not filtered. Now filter it.
				table.insert(chest_data.filtered_slots, slot)
				table.sort(chest_data.filtered_slots)
				element.style = "red_slot_button_style"
				-- Clear the slot.
				chest_data.entity.get_output_inventory()[slot].clear()
			end
		end
	end
end

-- Callback of the on_gui_text_changed event, which is invoked when the text of GUI element is changed by any player.
function gui.on_gui_text_changed(event)
	local element = event.element
	local element_name = element.name
	local player = game.players[element.player_index]
	local entity = global.creative_mode.player_opened_entities[element.player_index]
	
	if element_name == creative_mode_defines.names.gui.energy_source_set_output_textfield then
		-- Set energy source energy output.
		if entity then
			local energy_output = tonumber(element.text)
			if energy_output then
				entity.electric_output_flow_limit = energy_output
				element.text = entity.electric_output_flow_limit
			end
		end
	end
end